@@ -121,7 +121,7 @@ function insert_with_markers( $filename, $marker, $insertion ) {
 		if ( $perms ) {
 			chmod( $filename, $perms | 0644 );
 		}
-	} elseif ( ! is_writeable( $filename ) ) {
+	} elseif ( ! is_writable( $filename ) ) {
 		return false;
 	}
 
@@ -134,7 +134,7 @@ function insert_with_markers( $filename, $marker, $insertion ) {
 	$instructions = sprintf(
 		/* translators: 1: Marker. */
 		__(
-			'The directives (lines) between `BEGIN %1$s` and `END %1$s` are
+			'The directives (lines) between "BEGIN %1$s" and "END %1$s" are
 dynamically generated, and should only be modified via WordPress filters.
 Any changes to the directives between these markers will be overwritten.'
 		),
@@ -597,9 +597,11 @@ function wp_doc_link_parse( $content ) {
 			continue;
 		}
 
-		if ( T_STRING == $tokens[ $t ][0] && ( '(' == $tokens[ $t + 1 ] || '(' == $tokens[ $t + 2 ] ) ) {
+		if ( T_STRING == $tokens[ $t ][0] && ( '(' === $tokens[ $t + 1 ] || '(' === $tokens[ $t + 2 ] ) ) {
 			// If it's a function or class defined locally, there's not going to be any docs available.
-			if ( ( isset( $tokens[ $t - 2 ][1] ) && in_array( $tokens[ $t - 2 ][1], array( 'function', 'class' ) ) ) || ( isset( $tokens[ $t - 2 ][0] ) && T_OBJECT_OPERATOR == $tokens[ $t - 1 ][0] ) ) {
+			if ( ( isset( $tokens[ $t - 2 ][1] ) && in_array( $tokens[ $t - 2 ][1], array( 'function', 'class' ), true ) )
+				|| ( isset( $tokens[ $t - 2 ][0] ) && T_OBJECT_OPERATOR == $tokens[ $t - 1 ][0] )
+			) {
 				$ignore_functions[] = $tokens[ $t ][1];
 			}
 			// Add this to our stack of unique references.
@@ -623,7 +625,7 @@ function wp_doc_link_parse( $content ) {
 
 	$out = array();
 	foreach ( $functions as $function ) {
-		if ( in_array( $function, $ignore_functions ) ) {
+		if ( in_array( $function, $ignore_functions, true ) ) {
 			continue;
 		}
 		$out[] = $function;
@@ -656,9 +658,9 @@ function set_screen_options() {
 		$map_option = $option;
 		$type       = str_replace( 'edit_', '', $map_option );
 		$type       = str_replace( '_per_page', '', $type );
-		if ( in_array( $type, get_taxonomies() ) ) {
+		if ( in_array( $type, get_taxonomies(), true ) ) {
 			$map_option = 'edit_tags_per_page';
-		} elseif ( in_array( $type, get_post_types() ) ) {
+		} elseif ( in_array( $type, get_post_types(), true ) ) {
 			$map_option = 'edit_per_page';
 		} else {
 			$option = str_replace( '-', '_', $option );
@@ -686,23 +688,48 @@ function set_screen_options() {
 				}
 				break;
 			default:
+				$screen_option = false;
+
+				if ( '_page' === substr( $option, -5 ) || 'layout_columns' === $option ) {
+					/**
+					 * Filters a screen option value before it is set.
+					 *
+					 * The filter can also be used to modify non-standard [items]_per_page
+					 * settings. See the parent function for a full list of standard options.
+					 *
+					 * Returning false from the filter will skip saving the current option.
+					 *
+					 * @since 2.8.0
+					 * @since 5.4.2 Only applied to options ending with '_page',
+					 *              or the 'layout_columns' option.
+					 *
+					 * @see set_screen_options()
+					 *
+					 * @param mixed  $screen_option The value to save instead of the option value.
+					 *                              Default false (to skip saving the current option).
+					 * @param string $option        The option name.
+					 * @param int    $value         The option value.
+					 */
+					$screen_option = apply_filters( 'set-screen-option', $screen_option, $option, $value ); // phpcs:ignore WordPress.NamingConventions.ValidHookName.UseUnderscores
+				}
+
 				/**
 				 * Filters a screen option value before it is set.
 				 *
-				 * The filter can also be used to modify non-standard [items]_per_page
-				 * settings. See the parent function for a full list of standard options.
+				 * The dynamic portion of the hook, `$option`, refers to the option name.
 				 *
-				 * Returning false to the filter will skip saving the current option.
+				 * Returning false from the filter will skip saving the current option.
 				 *
-				 * @since 2.8.0
+				 * @since 5.4.2
 				 *
 				 * @see set_screen_options()
 				 *
-				 * @param bool     $keep   Whether to save or skip saving the screen option value. Default false.
-				 * @param string   $option The option name.
-				 * @param int      $value  The number of rows to use.
+				 * @param mixed   $screen_option The value to save instead of the option value.
+				 *                               Default false (to skip saving the current option).
+				 * @param string  $option        The option name.
+				 * @param int     $value         The option value.
 				 */
-				$value = apply_filters( 'set-screen-option', false, $option, $value ); // phpcs:ignore WordPress.NamingConventions.ValidHookName.UseUnderscores
+				$value = apply_filters( "set_screen_option_{$option}", $screen_option, $option, $value );
 
 				if ( false === $value ) {
 					return;
@@ -876,7 +903,7 @@ function iis7_add_rewrite_rule( $filename, $rewrite_rule ) {
  * @since 2.8.0
  *
  * @param DOMDocument $doc
- * @param string $filename
+ * @param string      $filename
  */
 function saveDomDocument( $doc, $filename ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid
 	$config = $doc->saveXML();
@@ -905,8 +932,9 @@ function admin_color_scheme_picker( $user_id ) {
 		$_wp_admin_css_colors = array_filter(
 			array_merge(
 				array(
-					'fresh' => '',
-					'light' => '',
+					'fresh'  => '',
+					'light'  => '',
+					'modern' => '',
 				),
 				$_wp_admin_css_colors
 			)
@@ -986,14 +1014,39 @@ function wp_color_scheme_settings() {
 }
 
 /**
- * @since 3.3.0
+ * Displays the viewport meta in the admin.
+ *
+ * @since 5.5.0
  */
-function _ipad_meta() {
-	if ( wp_is_mobile() ) {
-		?>
-		<meta name="viewport" id="viewport-meta" content="width=device-width, initial-scale=1">
-		<?php
+function wp_admin_viewport_meta() {
+	/**
+	 * Filters the viewport meta in the admin.
+	 *
+	 * @since 5.5.0
+	 *
+	 * @param string $viewport_meta The viewport meta.
+	 */
+	$viewport_meta = apply_filters( 'admin_viewport_meta', 'width=device-width,initial-scale=1.0' );
+
+	if ( empty( $viewport_meta ) ) {
+		return;
 	}
+
+	echo '<meta name="viewport" content="' . esc_attr( $viewport_meta ) . '">';
+}
+
+/**
+ * Adds viewport meta for mobile in Customizer.
+ *
+ * Hooked to the {@see 'admin_viewport_meta'} filter.
+ *
+ * @since 5.5.0
+ *
+ * @param string $viewport_meta The viewport meta.
+ * @return string Filtered viewport meta.
+ */
+function _customizer_mobile_viewport_meta( $viewport_meta ) {
+	return trim( $viewport_meta, ',' ) . ',minimum-scale=0.5,maximum-scale=1.2';
 }
 
 /**
@@ -1003,7 +1056,7 @@ function _ipad_meta() {
  *
  * @param array  $response  The Heartbeat response.
  * @param array  $data      The $_POST data sent.
- * @param string $screen_id The screen id.
+ * @param string $screen_id The screen ID.
  * @return array The Heartbeat response.
  */
 function wp_check_locked_posts( $response, $data, $screen_id ) {
@@ -1025,9 +1078,9 @@ function wp_check_locked_posts( $response, $data, $screen_id ) {
 						'text' => sprintf( __( '%s is currently editing' ), $user->display_name ),
 					);
 
-					$avatar = get_avatar( $user->ID, 18 );
-					if ( $avatar && preg_match( "|src='([^']+)'|", $avatar, $matches ) ) {
-						$send['avatar_src'] = $matches[1];
+					if ( get_option( 'show_avatars' ) ) {
+						$send['avatar_src']    = get_avatar_url( $user->ID, array( 'size' => 18 ) );
+						$send['avatar_src_2x'] = get_avatar_url( $user->ID, array( 'size' => 36 ) );
 					}
 
 					$checked[ $key ] = $send;
@@ -1050,7 +1103,7 @@ function wp_check_locked_posts( $response, $data, $screen_id ) {
  *
  * @param array  $response  The Heartbeat response.
  * @param array  $data      The $_POST data sent.
- * @param string $screen_id The screen id.
+ * @param string $screen_id The screen ID.
  * @return array The Heartbeat response.
  */
 function wp_refresh_post_lock( $response, $data, $screen_id ) {
@@ -1075,11 +1128,9 @@ function wp_refresh_post_lock( $response, $data, $screen_id ) {
 				'text' => sprintf( __( '%s has taken over and is currently editing.' ), $user->display_name ),
 			);
 
-			$avatar = get_avatar( $user->ID, 64 );
-			if ( $avatar ) {
-				if ( preg_match( "|src='([^']+)'|", $avatar, $matches ) ) {
-					$error['avatar_src'] = $matches[1];
-				}
+			if ( get_option( 'show_avatars' ) ) {
+				$error['avatar_src']    = get_avatar_url( $user->ID, array( 'size' => 64 ) );
+				$error['avatar_src_2x'] = get_avatar_url( $user->ID, array( 'size' => 128 ) );
 			}
 
 			$send['lock_error'] = $error;
@@ -1103,7 +1154,7 @@ function wp_refresh_post_lock( $response, $data, $screen_id ) {
  *
  * @param array  $response  The Heartbeat response.
  * @param array  $data      The $_POST data sent.
- * @param string $screen_id The screen id.
+ * @param string $screen_id The screen ID.
  * @return array The Heartbeat response.
  */
 function wp_refresh_post_nonces( $response, $data, $screen_id ) {
@@ -1139,7 +1190,7 @@ function wp_refresh_post_nonces( $response, $data, $screen_id ) {
  *
  * @since 5.0.0
  *
- * @param array  $response  The Heartbeat response.
+ * @param array $response The Heartbeat response.
  * @return array The Heartbeat response.
  */
 function wp_refresh_heartbeat_nonces( $response ) {
@@ -1195,7 +1246,7 @@ function heartbeat_autosave( $response, $data ) {
 				'message' => __( 'Error while saving.' ),
 			);
 		} else {
-			/* translators: Draft saved date format, see https://www.php.net/date */
+			/* translators: Draft saved date format, see https://www.php.net/manual/datetime.format.php */
 			$draft_saved_date_format = __( 'g:i:s a' );
 			$response['wp_autosave'] = array(
 				'success' => true,
@@ -1376,7 +1427,6 @@ All at ###SITENAME###
  *
  * @param string  $title Page title.
  * @param WP_Post $page  Page data object.
- *
  * @return string Page title.
  */
 function _wp_privacy_settings_filter_draft_page_titles( $title, $page ) {
@@ -1394,7 +1444,7 @@ function _wp_privacy_settings_filter_draft_page_titles( $title, $page ) {
  * @since 5.1.0
  * @since 5.1.1 Added the {@see 'wp_is_php_version_acceptable'} filter.
  *
- * @return array|false $response Array of PHP version data. False on failure.
+ * @return array|false Array of PHP version data. False on failure.
  */
 function wp_check_php_version() {
 	$version = phpversion();
